// 	WOSA/XRT Excel 5.0 AddIn Version 1.00
//
//	COPYRIGHT:
//
//  	(C) Copyright Microsoft Corp. 1994.  All rights reserved.
//
//   	You have a royalty-free right to use, modify, reproduce and
//   	distribute the Sample Files (and/or any modified version) in
//   	any way you find useful, provided that you agree that
//   	Microsoft has no warranty obligations or liability for any
//   	Sample Application Files which are modified.
//
//
// 	Revision History
//    	October 15, 1994   rc    Version 1.00 Release
//
// 	Comments:
//		The CDirtyCell implementation is used to kind of solve the problem
//	 	with xlSet. xlSet is very fast and can handle very large amounts of data.
//	 	However, xlSet does not provide a way to not update a cell which is within
//	 	the current update range. As a result, the normal form of updating is
//	 	to refresh the entire DataObject's range even when only one of the cells
//	 	change value. This can be particularly cumbersome when the needlessly
//	 	updated cell is refenced in a formula.
//	 
//		Another problem with updating the entire DataObject is forces the refresh
//		to update a range encompassing the entire DataObject even if the requests
//		within the DDataObject are of different sizes.
//
//		In any case, CDirtyCells provides some relief to these problems. CDirtyCells
//		will contain refences to cells whose values have changed. Do when DataObject.
//		GetData is called a separate xlSet is called for each cell change. Please
//		feel free to add further logic that groups adjacent dirty cells into a single
//		xlSet. Through some personal testing 5 xlSets was slower than updating an 
//		entire result set of over 1000 cells. However, as before mentioned if formulas 
//		refence these cells then DirtyCells is even more beneficial. 
//
//		The maximum number of DirtyCells allowed between GetData calls is 25. It can
//		be set by the VB developer through DataObject.UpdatePerCellCount.


#include "precomp.h"
#include "guid.h"
#include "dirtycel.h"

/*
 * CDirtyCells::Create
 *
 * Purpose:
 *  Creates an instance of a CDirtyCell and inits it
 *
 * Parameters
 *	ppDataObjects	Pointer to return create DataObjects
 *
 * Return Value
 *	HRESULT
 *
 */

HRESULT CDirtyCells::Create(CDirtyCells FAR* FAR* ppDirtyCells)
{
	CDirtyCells FAR *pDirtyCells = NULL;
	
	TRY{
		if(FAILED(CheckPointer((LPVOID) ppDirtyCells, 0))){
    		AddInDebugOutput(NULL, TEXT("CDirtyCells::Create")
    			, IDS_DEBUG_BADPOINTER, NULL); 
			return ResultFromScode(E_FAIL);
		}
	
    	*ppDirtyCells = NULL;
    	
		pDirtyCells = new CDirtyCells();
        
        if(pDirtyCells == NULL){
			AddInDebugOutput(NULL
				, TEXT("CDirtyCells::Create")
				, IDS_DEBUG_DIRTYCELLSALLOCFAILED
				, DISP_E_EXCEPTION); 
        	MYRAISEEXCEPTION;
        }
        
       	*ppDirtyCells = pDirtyCells;
	
		return S_OK;
	}
	
	
	EXCEPT{
   		AddInDebugOutput(NULL, TEXT("CDirtyCells::Create")
   			, IDS_DEBUG_EXCEPTION, NULL); 
	
    	if (pDirtyCells == NULL)
			return ResultFromScode(E_OUTOFMEMORY);
			          
		delete pDirtyCells;			          
			          
		return ResultFromScode(E_FAIL);
	}
}

/*
 * CDirtyCells::CDirtyCells
 *
 * Purpose:
 *  Constructor
 *
 */

CDirtyCells::CDirtyCells()
{
	m_nDirtyCellCount = 0;
	m_bRefresh = TRUE;
	m_nMaxCount = 1;
}

/*
 * CDirtyCells::~CDirtyCells
 *
 * Purpose:
 *  Destructor
 *
 */

CDirtyCells::~CDirtyCells()
{
}

/*
 * CDirtyCells::AddCell
 *
 * Purpose:
 *  Adds a dirty cell to collection
 *
 * Parameters
 *	nRow	Cell's row
 *	nCol	Cell's Col
 *
 * Return Value
 *	void
 *
 */

void CDirtyCells::AddCell(WORD wRow, WORD wCol)
{   
	TRY{
		if(!m_bRefresh){
			return;
		}
		
		// Ifcell updates is over Max then we cant do per cell refresh
		if(m_nDirtyCellCount > m_nMaxCount){
			m_bRefresh = FALSE;
			return;
		}
		
		// save cell
		m_pdwDirtyCellList[m_nDirtyCellCount++] = (DWORD) MAKELONG(wRow, wCol);

		return;
	}
	
	#ifdef _WIN32
	EXCEPT{
   		AddInDebugOutput(NULL, TEXT("CDirtyCells::AddCell")
   			, IDS_DEBUG_EXCEPTION, NULL); 
	
		return;
	}
	#endif									  
}

/*
 * CDirtyCells::Reset
 *
 * Purpose:
 *  Empties collection
 *
 * Return Value
 *	void
 *
 */

void CDirtyCells::Reset()
{
	m_nDirtyCellCount = 0;	
	
 	m_bRefresh = TRUE;
}

/*
 * CDirtyCells::GetCount
 *
 * Purpose:
 *  Get number of dirty cells in collection
 *
 * Return Value
 *	int
 *
 */

int CDirtyCells::GetCount()
{
	return m_nDirtyCellCount;
}

/*
 * CDirtyCells::GetCell
 *
 * Purpose:
 *  Returns a specific cell from collection
 *
 * Parameters
 *	index	ordinal index of the cell to return
 *
 * Return Value
 *	DWORD
 *
 */

DWORD CDirtyCells::GetCell(int index)
{        
	// If index is valid return it                    
	if(index < m_nDirtyCellCount){
		return m_pdwDirtyCellList[index];
	}
	
	return 0L;
}                      

/*
 * CDirtyCells::GetMaxCount
 *
 * Purpose:
 *  Returns the Max number of dirty cells allowed
 *
 * Return Value
 *	int
 *
 */

int CDirtyCells::GetMaxCount(){
	return m_nMaxCount;
}

/*
 * CDirtyCells::SetMaxCount
 *
 * Purpose:
 *  Sets the Max number of dirty cells allowed
 *
 * Return Value
 *	int
 *
 */

void CDirtyCells::SetMaxCount(int nValue){
	m_nMaxCount = nValue;
}

/*
 * CDirtyCells::IsSingleCellRefreshable
 *
 * Purpose:
 *  Called by DataObject.GetData to find out if it should do single cell
 *	update or whole DataObject update
 *
 * Return Value
 *	int
 *
 */

BOOL CDirtyCells::IsSingleCellRefreshable(){
	return m_bRefresh;
}

/*
 * CDirtyCells::SetNotSingleCellRefreshable
 *
 * Purpose:
 *	Turns off dirtycells for this update
 *
 * Return Value
 *	int
 *
 */

void CDirtyCells::SetNotSingleCellRefreshable(){
	m_bRefresh = FALSE;
}
